@@ -44,7 +44,6 @@ typedef enum {
 } Knowledge;
 
 typedef struct {
-	int automatic_emblems_as_integer;
 	char emblem_keywords[1];
 } NautilusFileSortByEmblemCache;
 
@@ -66,8 +65,6 @@ struct NautilusFileDetails
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult get_info_error;
 
-	NautilusMonitor *monitor;
-	
 	guint directory_count;
 
 	guint deep_directory_count;
@@ -83,6 +80,10 @@ struct NautilusFileDetails
 	char *custom_icon;
 	char *activation_uri;
 
+	/* The guessed (extension-based) mime type.  This is saved for
+	 * comparison vs. the slow mime type upon activation */
+	char *guessed_mime_type;
+
 	/* The following is for file operations in progress. Since
 	 * there are normally only a few of these, we can move them to
 	 * a separate hash table or something if required to keep the
@@ -150,7 +151,6 @@ struct NautilusFileDetails
 
 NautilusFile *nautilus_file_new_from_info                  (NautilusDirectory      *directory,
 							    GnomeVFSFileInfo       *info);
-NautilusFile *nautilus_file_get_existing                   (const char             *uri);
 void          nautilus_file_emit_changed                   (NautilusFile           *file);
 void          nautilus_file_mark_gone                      (NautilusFile           *file);
 gboolean      nautilus_file_info_missing                   (NautilusFile           *file,
