@@ -252,7 +252,7 @@ make_button_box (NautilusInformationPanel *information_panel)
 	gtk_widget_show (GTK_WIDGET (information_panel->details->button_box));
 	gtk_box_pack_start (GTK_BOX (information_panel->details->button_box_centerer),
 			    GTK_WIDGET (information_panel->details->button_box),
-			    TRUE, FALSE, 0);
+			    TRUE, TRUE, 0);
 	information_panel->details->has_buttons = FALSE;
 }
 
@@ -553,7 +553,6 @@ receive_dropped_uri_list (NautilusInformationPanel *information_panel,
 			eel_show_error_dialog (
 				_("You can't assign more than one custom icon at a time."),
 				_("Please drag just one image to set a custom icon."), 
-				_("More Than One Image"),
 				window);
 			break;
 		}
@@ -574,14 +573,12 @@ receive_dropped_uri_list (NautilusInformationPanel *information_panel,
 				eel_show_error_dialog (
 					_("The file that you dropped is not local."),
 					_("You can only use local images as custom icons."), 
-					_("Local Images Only"),
 					window);
 			
 			} else {
 				eel_show_error_dialog (
 					_("The file that you dropped is not an image."),
 					_("You can only use images as custom icons."),
-					_("Images Only"),
 					window);
 			}
 		}	
@@ -886,7 +883,7 @@ add_command_buttons (NautilusInformationPanel *information_panel, GList *applica
 {
 	char *id_string, *temp_str;
 	GList *p;
-	GtkWidget *temp_button;
+	GtkWidget *temp_button, *label;
 	GnomeVFSMimeApplication *application;
 
 	/* There's always at least the "Open with..." button */
@@ -897,6 +894,8 @@ add_command_buttons (NautilusInformationPanel *information_panel, GList *applica
 
 		temp_str = g_strdup_printf (_("Open with %s"), application->name);
 	        temp_button = gtk_button_new_with_label (temp_str);
+		label = GTK_BIN (temp_button)->child;
+		gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_START);
 		g_free (temp_str);
 		gtk_box_pack_start (GTK_BOX (information_panel->details->button_box), 
 				    temp_button, 
@@ -989,7 +988,6 @@ burn_cd_callback (GtkWidget *button, gpointer data)
 			    NULL,
 			    &error)) {
 		eel_show_error_dialog (_("Unable to launch the cd burner application."), error->message,
-				       _("Can't Launch CD Burner"),
 				       GTK_WINDOW (gtk_widget_get_toplevel (button)));
 		g_error_free (error);
 	}
