@@ -329,6 +329,15 @@ real_activate (NautilusNavigationBar *navigation_bar)
 	nautilus_entry_select_all (bar->details->entry);
 }
 
+static void
+real_cancel (NautilusNavigationBar *navigation_bar)
+{
+	char *last_location;
+
+	last_location = NAUTILUS_LOCATION_BAR (navigation_bar)->details->last_location;
+	nautilus_navigation_bar_set_location (navigation_bar, last_location);
+}
+
 static void
 finalize (GObject *object)
 {
@@ -384,6 +393,7 @@ nautilus_location_bar_class_init (NautilusLocationBarClass *class)
 	navigation_bar_class = NAUTILUS_NAVIGATION_BAR_CLASS (class);
 
 	navigation_bar_class->activate = real_activate;
+	navigation_bar_class->cancel = real_cancel;
 	navigation_bar_class->get_location = nautilus_location_bar_get_location;
 	navigation_bar_class->set_location = nautilus_location_bar_set_location;
 }
@@ -487,12 +497,17 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 
 	/* Note: This is called in reaction to external changes, and 
 	 * thus should not emit the LOCATION_CHANGED signal. */
-	
-	formatted_location = eel_format_uri_for_display (location);
-	nautilus_entry_set_text (NAUTILUS_ENTRY (bar->details->entry),
-				 formatted_location);
-	set_position_and_selection_to_end (GTK_EDITABLE (bar->details->entry));
-	g_free (formatted_location);
+
+	if (eel_uri_is_search (location)) {
+		nautilus_location_entry_set_special_text (NAUTILUS_LOCATION_ENTRY (bar->details->entry),
+							  "");
+	} else {
+		formatted_location = eel_format_uri_for_display (location);
+		nautilus_entry_set_text (NAUTILUS_ENTRY (bar->details->entry),
+					 formatted_location);
+		set_position_and_selection_to_end (GTK_EDITABLE (bar->details->entry));
+		g_free (formatted_location);
+	}
 
 	/* free up the cached file info from the previous location */
 	g_free (bar->details->current_directory);
@@ -503,8 +518,11 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 	
 	/* remember the original location for later comparison */
 	
-	g_free (bar->details->last_location);
-	bar->details->last_location = g_strdup (location);
+	if (bar->details->last_location != location) {
+		g_free (bar->details->last_location);
+		bar->details->last_location = g_strdup (location);
+	}
+
 	nautilus_location_bar_update_label (bar);
 }
 
