@@ -38,6 +38,8 @@
 #include "nautilus-signaller.h"
 #include "nautilus-location-bar.h"
 #include "nautilus-pathbar.h"
+#include "nautilus-query-editor.h"
+#include "nautilus-search-bar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
 #include <eel/eel-accessibility.h>
@@ -48,15 +50,18 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
+#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkmenuitem.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtktreemodel.h>
+#include <gtk/gtkliststore.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
@@ -79,6 +84,7 @@
 #include <libnautilus-private/nautilus-undo.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
+#include <libnautilus-private/nautilus-search-directory.h>
 #include <math.h>
 #include <sys/time.h>
 
@@ -97,6 +103,12 @@
 
 #define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/MenuBar/Other Menus/Bookmarks/Bookmarks Placeholder"
 
+typedef enum {
+	NAUTILUS_BAR_PATH,
+	NAUTILUS_BAR_NAVIGATION,
+	NAUTILUS_BAR_SEARCH
+} NautilusBarMode;
+
 enum {
 	ARG_0,
 	ARG_APP_ID,
@@ -119,6 +131,14 @@ static void path_bar_location_changed_callback       (GtkWidget                *
 						      NautilusNavigationWindow *window);
 static void always_use_location_entry_changed        (gpointer                  callback_data);
 
+static void nautilus_navigation_window_set_bar_mode  (NautilusNavigationWindow *window, 
+						      NautilusBarMode           mode);
+static void search_bar_activate_callback             (NautilusSearchBar        *bar,
+						      NautilusWindow           *window);
+static void search_bar_cancel_callback               (GtkWidget                *widget,
+						      NautilusNavigationWindow *window);
+
+static void nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWindow *window);
 
 GNOME_CLASS_BOILERPLATE (NautilusNavigationWindow, nautilus_navigation_window,
 			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
@@ -131,7 +151,7 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	GtkWidget *location_bar;
 	GtkWidget *view_as_menu_vbox;
 	GtkToolItem *item;
-	GtkWidget *hbox;
+	GtkWidget *hbox, *vbox, *eventbox, *extras_vbox;
 	
 	window->details = g_new0 (NautilusNavigationWindowDetails, 1);
 
@@ -148,6 +168,25 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 			  0,                                  0);
 	gtk_widget_show (window->details->content_paned);
 
+	vbox = gtk_vbox_new (FALSE, 0);
+	nautilus_horizontal_splitter_pack2 (
+		NAUTILUS_HORIZONTAL_SPLITTER (window->details->content_paned),
+		vbox);
+	gtk_widget_show (vbox);
+
+	eventbox = gtk_event_box_new ();
+	gtk_widget_set_name (eventbox, "nautilus-extra-view-widget");
+	gtk_box_pack_start (GTK_BOX (vbox), eventbox, FALSE, FALSE, 0);
+	gtk_widget_show (eventbox);
+	
+	extras_vbox = gtk_vbox_new (FALSE, 0);
+	NAUTILUS_WINDOW (window)->details->extra_location_widgets = extras_vbox;
+	gtk_container_add (GTK_CONTAINER (eventbox), extras_vbox);
+
+	window->details->content_box = gtk_vbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), window->details->content_box, TRUE, TRUE, 0);
+	gtk_widget_show (window->details->content_box);
+	
 	nautilus_navigation_window_initialize_actions (window);
 	nautilus_navigation_window_initialize_menus (window);
 	
@@ -205,6 +244,15 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 			    window->navigation_bar,
 			    TRUE, TRUE, 0);
 
+	window->search_bar = nautilus_search_bar_new ();
+	g_signal_connect_object (window->search_bar, "activate",
+				 G_CALLBACK (search_bar_activate_callback), window, 0);
+	g_signal_connect_object (window->search_bar, "cancel",
+				 G_CALLBACK (search_bar_cancel_callback), window, 0);
+	gtk_box_pack_start (GTK_BOX (hbox),
+			    window->search_bar,
+			    TRUE, TRUE, 0);
+
 	/* Option menu for content view types; it's empty here, filled in when a uri is set.
 	 * Pack it into vbox so it doesn't grow vertically when location bar does. 
 	 */
@@ -218,9 +266,9 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	gtk_toolbar_insert (GTK_TOOLBAR (location_bar),
 			    item, -1);
 	
-	window->view_as_option_menu = gtk_option_menu_new ();
-	gtk_box_pack_end (GTK_BOX (view_as_menu_vbox), window->view_as_option_menu, TRUE, FALSE, 0);
-	gtk_widget_show (window->view_as_option_menu);
+	window->view_as_combo_box = gtk_combo_box_new_text ();
+	gtk_box_pack_end (GTK_BOX (view_as_menu_vbox), window->view_as_combo_box, TRUE, FALSE, 0);
+	gtk_widget_show (window->view_as_combo_box);
 
 	/* Allocate the zoom control and place on the right next to the menu.
 	 * It gets shown later, if the view-frame contains something zoomable.
@@ -268,9 +316,9 @@ always_use_location_entry_changed (gpointer callback_data)
 	window = NAUTILUS_NAVIGATION_WINDOW (callback_data);
 
 	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
-		nautilus_navigation_window_hide_path_bar (window);
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
 	} else {
-		nautilus_navigation_window_show_path_bar (window);
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
 	}
 }
 
@@ -329,11 +377,23 @@ hide_temporary_bars (NautilusNavigationWindow *window)
 		window->details->temporary_location_bar = FALSE;
 	}
 	if (window->details->temporary_navigation_bar) {
-		if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
-			nautilus_navigation_window_show_path_bar (window);
+		if (NAUTILUS_WINDOW (window)->details->search_mode) {
+			nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_SEARCH);
+		} else {
+			if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+				nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
+			}
 		}
 		window->details->temporary_navigation_bar = FALSE;
 	}
+	if (window->details->temporary_search_bar) {
+		if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+			nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
+		} else {
+			nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
+		}
+		window->details->temporary_search_bar = FALSE;
+	}
 }
 
 static void
@@ -518,7 +578,7 @@ nautilus_navigation_window_destroy (GtkObject *object)
 	g_list_foreach (window->sidebar_panels, (GFunc)g_object_unref, NULL);
 	window->sidebar_panels = NULL;
 
-	window->view_as_option_menu = NULL;
+	window->view_as_combo_box = NULL;
 	window->navigation_bar = NULL;
 	window->path_bar = NULL;
 	window->zoom_control = NULL;
@@ -688,88 +748,69 @@ activate_extra_viewer (NautilusWindow *window)
 }
 
 static void
-view_as_menu_switch_views_callback (GtkWidget *widget, gpointer data)
-{
-        NautilusWindow *window;
-        int viewer_index;
-        
-        g_assert (GTK_IS_MENU_ITEM (widget));
-        g_assert (NAUTILUS_IS_WINDOW (data));
+view_as_menu_switch_views_callback (GtkComboBox *combo_box, NautilusWindow *window)
+{         
+	int active;
+	g_assert (GTK_IS_COMBO_BOX (combo_box));
+	g_assert (NAUTILUS_IS_WINDOW (window));
 
-        window = NAUTILUS_WINDOW (data);
-
-        if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "extra viewer")) == TRUE) {
-        	activate_extra_viewer (window);
-        } else {
-		viewer_index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "viewer index"));
-		activate_nth_short_list_item (window, viewer_index);
-        }
-}
-
-static GtkWidget *
-create_view_as_menu_item (NautilusWindow *window, 
-			  const char *identifier,
-			  guint index)
-{
-	GtkWidget *menu_item;
-	const NautilusViewInfo *info;
-
-	info = nautilus_view_factory_lookup (identifier);
-
-	menu_item = gtk_menu_item_new_with_mnemonic (_(info->view_as_label));
-
-	g_signal_connect_object (menu_item, "activate",
-				 G_CALLBACK (view_as_menu_switch_views_callback),
-				 window, 0);
-
-	g_object_set_data (G_OBJECT (menu_item), "viewer index", GINT_TO_POINTER (index));
-
-	gtk_widget_show (menu_item);
-
-	return menu_item;
+	active = gtk_combo_box_get_active (combo_box);
+	
+	if (active < GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combo_box), "num viewers"))  ) {
+		activate_nth_short_list_item (window, active);
+	} else {
+		activate_extra_viewer (window);
+	}
 }
 
 static void
 load_view_as_menu (NautilusWindow *window)
 {
-	GtkWidget *new_menu;
-	GtkWidget *menu_item;
 	GList *node;
 	int index;
 	int selected_index = -1;
-
-        new_menu = gtk_menu_new ();
+	GtkTreeModel *model;
+	GtkListStore *store;
+	const NautilusViewInfo *info;
+	GtkComboBox* combo_box;
 	
+    	combo_box = GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box);
+	/* Clear the contents of ComboBox in a wacky way because there
+	 * is no function to clear all items and also no function to obtain
+	 * the number of items in a combobox.
+	 */
+	model = gtk_combo_box_get_model (combo_box);
+	g_return_if_fail (GTK_IS_LIST_STORE (model));
+	store = GTK_LIST_STORE (model);
+	gtk_list_store_clear (store);
+
         /* Add a menu item for each view in the preferred list for this location. */
         for (node = window->details->short_list_viewers, index = 0; 
              node != NULL; 
              node = node->next, ++index) {
-                menu_item = create_view_as_menu_item (window, node->data, index);
-                gtk_menu_shell_append (GTK_MENU_SHELL (new_menu), menu_item);
+		info = nautilus_view_factory_lookup (node->data);
+		gtk_combo_box_append_text (combo_box, _(info->view_as_label));
 
 		if (nautilus_window_content_view_matches_iid (NAUTILUS_WINDOW (window), (char *)node->data)) {
 			selected_index = index;
 		}
         }
-
+	g_object_set_data (G_OBJECT (combo_box), "num viewers", GINT_TO_POINTER (index));
 	if (selected_index == -1) {
 		const char *id;
 		/* We're using an extra viewer, add a menu item for it */
 
 		id = nautilus_window_get_content_view_id (window);
-                menu_item = create_view_as_menu_item (window, id, index);
-		
-                gtk_menu_shell_append (GTK_MENU_SHELL (new_menu), menu_item);
+		info = nautilus_view_factory_lookup (id);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box),
+					   _(info->view_as_label));
 		selected_index = index;
 	}
 
-        /* We create and attach a new menu here because adding/removing
-         * items from existing menu screws up the size of the option menu.
-         */
-        gtk_option_menu_set_menu (GTK_OPTION_MENU (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_option_menu),
-                                  new_menu);
+	g_signal_connect (GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box), 
+			  "changed", G_CALLBACK (view_as_menu_switch_views_callback), window);
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_option_menu), selected_index);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box), selected_index);
 }
 
 static void
@@ -860,9 +901,8 @@ real_set_content_view_widget (NautilusWindow *nautilus_window,
 
 	connect_view (window, new_view);
 
-	nautilus_horizontal_splitter_pack2 (
-		NAUTILUS_HORIZONTAL_SPLITTER (window->details->content_paned),
-		GTK_WIDGET (new_view));
+	gtk_container_add (GTK_CONTAINER (window->details->content_box),
+			   GTK_WIDGET (new_view));
 
 	if (new_view != NULL && nautilus_view_supports_zooming (new_view)) {
 		gtk_widget_show (window->zoom_control);
@@ -888,15 +928,20 @@ real_set_throbber_active (NautilusWindow *window, gboolean active)
 }
 
 static void
-nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWindow *window, 
-							  gboolean in_search_mode)
+nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWindow *window)
 {
 	if (!nautilus_navigation_window_location_bar_showing (window)) {
 		nautilus_navigation_window_show_location_bar (window, FALSE);
 		window->details->temporary_location_bar = TRUE;
 	}
-	if (nautilus_navigation_window_path_bar_showing (window)) {
-		nautilus_navigation_window_hide_path_bar (window);
+}
+
+static void
+nautilus_navigation_window_show_navigation_bar_temporarily (NautilusNavigationWindow *window)
+{
+	if (nautilus_navigation_window_path_bar_showing (window)
+	    || nautilus_navigation_window_search_bar_showing (window)) {
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
 		window->details->temporary_navigation_bar = TRUE;
 	}
 	nautilus_navigation_bar_activate 
@@ -906,13 +951,132 @@ nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWind
 static void
 real_prompt_for_location (NautilusWindow *window, const char *initial)
 {
-	nautilus_navigation_window_show_location_bar_temporarily (NAUTILUS_NAVIGATION_WINDOW (window), FALSE);
-
+	nautilus_navigation_window_show_location_bar_temporarily (NAUTILUS_NAVIGATION_WINDOW (window));
+	nautilus_navigation_window_show_navigation_bar_temporarily (NAUTILUS_NAVIGATION_WINDOW (window));
+	
 	if (initial) {
 		nautilus_navigation_bar_set_location (NAUTILUS_NAVIGATION_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->navigation_bar),
 						      initial);
 	}
-		
+}
+
+static void
+search_bar_activate_callback (NautilusSearchBar *bar,
+			      NautilusWindow *window)
+{
+	char *uri, *home_uri;
+	NautilusDirectory *directory;
+	NautilusSearchDirectory *search_directory;
+	NautilusQuery *query;
+
+	uri = nautilus_search_directory_generate_new_uri ();
+	directory = nautilus_directory_get (uri);
+	
+	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
+
+	search_directory = NAUTILUS_SEARCH_DIRECTORY (directory);
+
+	query = nautilus_search_bar_get_query (NAUTILUS_SEARCH_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->search_bar));
+	if (query != NULL) {
+		if (!nautilus_search_directory_is_indexed (search_directory)) {
+			home_uri = nautilus_get_home_directory_uri ();
+			nautilus_query_set_location (query, home_uri);
+			g_free (home_uri);
+		}
+		nautilus_search_directory_set_query (search_directory, query);
+		g_object_unref (query);
+	}
+	
+	nautilus_window_go_to (window, uri);
+	
+	nautilus_directory_unref (directory);
+	g_free (uri);
+}
+
+static void
+search_bar_cancel_callback (GtkWidget *widget,
+			    NautilusNavigationWindow *window)
+{
+	hide_temporary_bars (window);
+}
+
+void 
+nautilus_navigation_window_show_search (NautilusNavigationWindow *window)
+{
+	if (!nautilus_navigation_window_search_bar_showing (window)) {
+		nautilus_navigation_window_show_location_bar_temporarily (window);
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_SEARCH);
+		window->details->temporary_search_bar = TRUE;
+		nautilus_search_bar_clear (NAUTILUS_SEARCH_BAR (window->search_bar));
+	}
+	
+	nautilus_search_bar_grab_focus (NAUTILUS_SEARCH_BAR (window->search_bar));
+}
+
+static void
+query_editor_changed_callback (NautilusSearchBar *bar,
+			       NautilusQuery *query,
+			       gboolean reload,
+			       NautilusWindow *window)
+{
+	NautilusDirectory *directory;
+
+	directory = nautilus_directory_get_for_file (window->details->viewed_file);
+	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
+
+	nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
+					     query);
+	if (reload) {
+		nautilus_window_reload (window);
+	}
+
+	nautilus_directory_unref (directory);
+}
+
+static void
+real_set_search_mode (NautilusWindow *window, gboolean search_mode,
+		      NautilusSearchDirectory *search_directory)
+{
+	NautilusNavigationWindow *nav_window;
+	GtkWidget *query_editor;
+	NautilusQuery *query;
+
+	nav_window = NAUTILUS_NAVIGATION_WINDOW (window);
+
+	if (!search_mode) {
+		nav_window->details->temporary_search_bar = TRUE;
+		hide_temporary_bars (nav_window);
+		return;
+	}
+
+	if (nautilus_search_directory_is_saved_search (search_directory)) {
+		query_editor = nautilus_query_editor_new (TRUE,
+							  nautilus_search_directory_is_indexed (search_directory));
+	} else {
+		nautilus_navigation_window_show_location_bar_temporarily (nav_window);
+		nautilus_navigation_window_set_bar_mode (nav_window, NAUTILUS_BAR_SEARCH);
+		nav_window->details->temporary_search_bar = FALSE;
+
+		query_editor = nautilus_query_editor_new_with_bar (FALSE,
+								   nautilus_search_directory_is_indexed (search_directory),
+								   NAUTILUS_SEARCH_BAR (nav_window->search_bar));
+	}
+
+	g_signal_connect_object (query_editor, "changed",
+				 G_CALLBACK (query_editor_changed_callback), window, 0);
+	
+	query = nautilus_search_directory_get_query (search_directory);
+	if (query != NULL) {
+		nautilus_query_editor_set_query (NAUTILUS_QUERY_EDITOR (query_editor),
+						 query);
+		g_object_unref (query);
+	}else {
+		nautilus_query_editor_set_default_query (NAUTILUS_QUERY_EDITOR (query_editor));
+	}
+	
+	nautilus_window_add_extra_location_widget (window, query_editor);
+	gtk_widget_show (query_editor);
+	nautilus_query_editor_grab_focus (NAUTILUS_QUERY_EDITOR (query_editor));
 }
 
 void
@@ -1021,19 +1185,40 @@ nautilus_navigation_window_location_bar_showing (NautilusNavigationWindow *windo
 	return TRUE;
 }
 
-void 
-nautilus_navigation_window_hide_path_bar (NautilusNavigationWindow *window)
+gboolean
+nautilus_navigation_window_search_bar_showing (NautilusNavigationWindow *window)
 {
-	window->details->temporary_navigation_bar = FALSE;
-	gtk_widget_hide (window->path_bar);
-	gtk_widget_show (window->navigation_bar);
+	if (window->search_bar != NULL) {
+		return GTK_WIDGET_VISIBLE (window->search_bar);
+	}
+	/* If we're not visible yet we haven't changed visibility, so its TRUE */
+	return TRUE;
 }
 
-void 
-nautilus_navigation_window_show_path_bar (NautilusNavigationWindow *window)
+static void
+nautilus_navigation_window_set_bar_mode (NautilusNavigationWindow *window,
+					 NautilusBarMode mode)
 {
-	gtk_widget_show (window->path_bar);
-	gtk_widget_hide (window->navigation_bar);
+	switch (mode) {
+
+	case NAUTILUS_BAR_PATH:
+		gtk_widget_show (window->path_bar);
+		gtk_widget_hide (window->navigation_bar);
+		gtk_widget_hide (window->search_bar);
+		break;
+
+	case NAUTILUS_BAR_NAVIGATION:
+		gtk_widget_show (window->navigation_bar);
+		gtk_widget_hide (window->path_bar);
+		gtk_widget_hide (window->search_bar);
+		break;
+
+	case NAUTILUS_BAR_SEARCH:
+		gtk_widget_show (window->search_bar);
+		gtk_widget_hide (window->path_bar);
+		gtk_widget_hide (window->navigation_bar);
+		break;
+	}
 }
 
 gboolean
@@ -1213,9 +1398,9 @@ nautilus_navigation_window_show (GtkWidget *widget)
 	}
 
 	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
-		nautilus_navigation_window_hide_path_bar (window);
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_NAVIGATION);
 	} else {
-		nautilus_navigation_window_show_path_bar (window);
+		nautilus_navigation_window_set_bar_mode (window, NAUTILUS_BAR_PATH);
 	}
 	
 	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR)) {
@@ -1288,8 +1473,9 @@ nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 	NAUTILUS_WINDOW_CLASS (class)->set_content_view_widget = real_set_content_view_widget;
 	NAUTILUS_WINDOW_CLASS (class)->set_throbber_active = real_set_throbber_active;
 	NAUTILUS_WINDOW_CLASS (class)->prompt_for_location = real_prompt_for_location;
+	NAUTILUS_WINDOW_CLASS (class)->set_search_mode = real_set_search_mode;
 	NAUTILUS_WINDOW_CLASS (class)->set_title = real_set_title;
 	NAUTILUS_WINDOW_CLASS (class)->get_icon_name = real_get_icon_name;
-	NAUTILUS_WINDOW_CLASS(class)->get_default_size = real_get_default_size;
+	NAUTILUS_WINDOW_CLASS (class)->get_default_size = real_get_default_size;
 	NAUTILUS_WINDOW_CLASS (class)->close = real_window_close;
 }
