@@ -31,6 +31,7 @@
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
 #include "nautilus-location-bar.h"
+#include "nautilus-search-bar.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-main.h"
 #include "nautilus-window-private.h"
@@ -59,6 +60,7 @@
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-monitor.h>
+#include <libnautilus-private/nautilus-search-directory.h>
 #include <libnautilus-private/nautilus-view-factory.h>
 #include <libnautilus-private/nautilus-window-info.h>
 
@@ -104,6 +106,8 @@ static void location_has_really_changed               (NautilusWindow
 static void update_for_new_location                   (NautilusWindow             *window);
 static void zoom_parameters_changed_callback          (NautilusView               *view,
 						       NautilusWindow             *window);
+static void update_extra_location_widgets_visibility  (NautilusWindow             *window);
+static void remove_extra_location_widgets             (NautilusWindow             *window);
 
 void
 nautilus_window_report_selection_changed (NautilusWindowInfo *window)
@@ -607,7 +611,6 @@ report_current_content_view_failure_to_user (NautilusWindow *window,
 	message = nautilus_window_get_view_startup_error_label (window);
 	eel_show_error_dialog (message,
 			       _("You can choose another view or go to a different location."),
-			       _("View Failed"),
 			       GTK_WINDOW (window));
 	g_free (message);
 }
@@ -621,7 +624,6 @@ report_nascent_content_view_failure_to_user (NautilusWindow *window,
 	message = nautilus_window_get_view_error_label (window);
 	eel_show_error_dialog (message,
 			       _("The location cannot be displayed with this viewer."),
-			       _("View Failed"),
 			       GTK_WINDOW (window));
 	g_free (message);
 }
@@ -1141,6 +1143,8 @@ update_for_new_location (NautilusWindow *window)
 {
         char *new_location;
         NautilusFile *file;
+	NautilusDirectory *directory;
+	gboolean location_really_changed;
         
         new_location = window->details->pending_location;
         window->details->pending_location = NULL;
@@ -1149,6 +1153,8 @@ update_for_new_location (NautilusWindow *window)
 
         update_history (window, window->details->location_change_type, new_location);
                 
+	location_really_changed = eel_strcmp (window->details->location, new_location) != 0;
+		
         /* Set the new location. */
         g_free (window->details->location);
         window->details->location = new_location;
@@ -1178,6 +1184,20 @@ update_for_new_location (NautilusWindow *window)
 	/* Load menus from nautilus extensions for this location */
 	nautilus_window_load_extension_menus (window);
 
+	if (location_really_changed) {
+		remove_extra_location_widgets (window);
+		
+		directory = nautilus_directory_get (window->details->location);
+		if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+			nautilus_window_set_search_mode (window, TRUE, NAUTILUS_SEARCH_DIRECTORY (directory));
+		} else {
+			nautilus_window_set_search_mode (window, FALSE, NULL);
+		}
+		nautilus_directory_unref (directory);
+
+		update_extra_location_widgets_visibility (window);
+	}
+
 #if !NEW_UI_COMPLETE
         if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
                 /* Check if the back and forward buttons need enabling or disabling. */
@@ -1342,7 +1362,6 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 	char *error_message;
 	char *detail_message;
 	char *scheme_string;
-	char *dialog_title;
 	const char *host_name;
 	GtkDialog *dialog;
 	GnomeVFSURI *vfs_uri;
@@ -1358,9 +1377,7 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 	uri_for_display = eel_str_middle_truncate
 		(full_uri_for_display, MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_uri_for_display);
-	
-	dialog_title = NULL;
-        
+	        
 	switch (result_code) {
 	case GNOME_VFS_OK:
 		if (nautilus_file_is_directory (file)) {
@@ -1458,13 +1475,8 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 		detail_message = g_strdup (_("Please select another viewer and try again."));
 	}
 	
-	if (dialog_title == NULL) {
-		dialog_title = g_strdup (_("Can't Display Location"));
-	}
-
-	dialog = eel_show_error_dialog (error_message, detail_message, dialog_title, NULL);
+	dialog = eel_show_error_dialog (error_message, detail_message, NULL);
 	
-	g_free (dialog_title);
 	g_free (uri_for_display);
 	g_free (error_message);
 	g_free (detail_message);
@@ -1711,3 +1723,44 @@ nautilus_window_reload (NautilusWindow *window)
 	g_free (location);
 	eel_g_list_free_deep (selection);
 }
+
+static void
+remove_all (GtkWidget *widget,
+	    gpointer data)
+{
+	GtkContainer *container;
+	container = GTK_CONTAINER (data);
+
+	gtk_container_remove (container, widget);
+}
+
+static void
+remove_extra_location_widgets (NautilusWindow *window)
+{
+	gtk_container_foreach (GTK_CONTAINER (window->details->extra_location_widgets),
+			       remove_all,
+			       window->details->extra_location_widgets);
+}
+
+void
+nautilus_window_add_extra_location_widget (NautilusWindow *window,
+					   GtkWidget *widget)
+{
+	gtk_box_pack_start (GTK_BOX (window->details->extra_location_widgets),
+			    widget, TRUE, TRUE, 0);
+}
+
+static void
+update_extra_location_widgets_visibility (NautilusWindow *window)
+{
+	GList *children;
+	
+	children = gtk_container_get_children (GTK_CONTAINER (window->details->extra_location_widgets));
+	
+	if (children != NULL) {
+		gtk_widget_show (window->details->extra_location_widgets);
+	} else {
+		gtk_widget_show (window->details->extra_location_widgets);
+	}
+	g_list_free (children);
+}
